////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Merry Hexmas/Camelot
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Code: Cruzer/Camelot^F*A*I*C
// Tune/player: Scarzix/Offence^F*A*I*C
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Asm: KickAss
// Tab size: 8
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
.import source "cruzersLib.asm"
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
.var cnt		= $02
.var basicOutput	= $65
.var playDelay		= basicOutput
.var sine 		= $0700
.var basic		= $0801
.var sprite		= $0cc0
.var rndSrc 		= $e600
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
.var voice = $d40e
.var wave = 4
.var volume = 10
.var waveform = $11
.var freqMul = 3.0
.var holidayGreetingCardMode = f
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
:basic($080d)

.pc = * "code"

	sei

//generate sine using basic/kernal ROM. Based on routine by Doynax

.var bias = $befa

loop:	lda #<index
	ldy #>index
	jsr $bba2
	jsr $e277
	lda #<bias
	ldy #>bias
	jsr $b867
	lda basicOutput
	eor #$80
	lsr
index:
	.byte $90
	.byte $00
sts:	sta sine
	inc sts+1
	bne loop


//init vic & sid...

	ldx #volume
!:	
	lda #$09
	sta voice,x 
	lda #$00
	sta sprite,x
	sta $d020
	sta $d021
	dex
	bpl !-

	inc $d015
	inc $d017
	inc $d01d
	inc sprite + 2	//set pixel

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
mainLoop:

	//open lower border and keep screen turned off to avoid badlines...
	
	lda #$fa
	ldx #$20
	jsr rasterCmp
	
	txa
	tay
	ldx #$09
	jsr rasterCmp

	lda #$16
	sta $d001
	jsr rasterCmp
		
rasterLoop:
	
	lda cnt
	adc rndSrc,y
	anc #$7f
	tax
	
	lda #$10
	sta $d017
ls0:	adc sine + $00,x
ls1:	adc sine + $81,x
	sta $d000
	
	inc $d017
	
	tya
	and #$07
	bne keepWidth

incWidth:
	inc ls0+1
	dec ls1+1
	
	iny
	bne rasterLoop
	
keepWidth:
	nop
	ora ($02,x)	//waste 6 cycles
	sty $d027
	
!:	iny
	bne rasterLoop


	sty $d027	//sprite color = black
	
	sty ls0+1
	lda #$81
	sta ls1+1

	inc cnt

play:
	.var patternSize = patternEnd - pattern0

	dec playDelay
	bpl done	

	dec [voice+wave]		//gate off
	
cursor:	ldy #patternSize	
	dey
	bpl !+
	ldy #patternSize
!:	sty cursor+1
	
	lax pattern0,y
	and #%00000111
	tay
	
	lda freqLo,y
	sta voice+0
	lda freqHi,y
	sta voice+1
	
	lda #$f8
	sax playDelay
	
	lda #waveform		//gate on
	sta [voice+wave]

done:
.print "mainLoop: $" + hex(* - mainLoop)

	jmp mainLoop


//tune data...

.var freqs = List().add($171c,$19f0,$1d1e,$1ed9,$22a0)

.for (var i=0; i<freqs.size(); i++) {
	.var newFreq = freqs.get(i) * freqMul * [holidayGreetingCardMode ? [1.1 - i*0.02] :1]
	.print "freq: $" + hex(newFreq)
	.eval freqs.set(i, newFreq)
}

freqLo:	:dumpLo(freqs)
freqHi:	:dumpHi(freqs)


.enum {F_4, G_4, A_4, Ax4, C_5}

.macro sound(note, delay) {
	.by [delay <<3] | note 
}


//notes in reverse order...

pattern0:

:sound(C_5, 4)
:sound(G_4, 4)

:sound(A_4, 2)
:sound(G_4, 2)
:sound(G_4, 2)
:sound(A_4, 2)

:sound(Ax4, 1)
:sound(A_4, 3)
:sound(A_4, 2)
:sound(Ax4, 2)

:sound(C_5, 1)
:sound(Ax4, 3)
:sound(Ax4, 2)
:sound(Ax4, 2)

:sound(A_4, 8)
:sound(G_4, 1)
:sound(F_4, 3)
:sound(C_5, 2)
:sound(A_4, 2)

:sound(A_4, 4)
:sound(A_4, 2)
:sound(A_4, 2)

:sound(A_4, 4)
:sound(A_4, 2)
:sound(A_4, 2)

patternEnd:


rasterCmp:
!:	cmp $d012
	bne !-
	stx $d011
	rts


.var size = * - $0801
.print "size: $" + hex(size)

